@@ -62,6 +62,7 @@
 #include "../jrd/gds_proto.h"
 #include "../jrd/err_proto.h"
 #include "../jrd/intl_proto.h"
+#include "../jrd/nbak.h"
 
 using namespace Jrd;
 
@@ -102,10 +103,19 @@ int INF_blob_info(const blb* blob,
  *
  **************************************/
 	SCHAR buffer[128];
-	SSHORT length;
+	USHORT length;
 
 	const SCHAR* const end_items = items + item_length;
 	const SCHAR* const end = info + output_length;
+	SCHAR* start_info;
+	
+	if (*items == isc_info_length) {
+		start_info = info;
+		items++;
+	}
+	else {
+		start_info = 0;
+	}
 
 	while (items < end_items && *items != isc_info_end) {
 		SCHAR item = *items++;
@@ -142,6 +152,14 @@ int INF_blob_info(const blb* blob,
 
 	*info++ = isc_info_end;
 
+	if (start_info && (end - info >= 7))
+	{
+		SLONG number = info - start_info;
+		memmove(start_info + 7, start_info, number);
+		length = INF_convert(number, buffer);
+		INF_put_item(isc_info_length, length, buffer, start_info, end);
+	}
+
 	return TRUE;
 }
 
@@ -162,9 +180,7 @@ USHORT INF_convert(SLONG number, SCHAR* buffer)
 	const SCHAR* p;
 
 #ifndef WORDS_BIGENDIAN
-	// CVC: What's the need for an intermediate "n" here?
-	const SLONG n = number;
-	p = reinterpret_cast<const SCHAR*>(&n);
+	p = reinterpret_cast<const SCHAR*>(&number);
 	*buffer++ = *p++;
 	*buffer++ = *p++;
 	*buffer++ = *p++;
@@ -206,7 +222,7 @@ int INF_database_info(const SCHAR* items,
 	bool header_refreshed = false;
 
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 	jrd_tra* transaction = NULL;
@@ -214,8 +230,7 @@ int INF_database_info(const SCHAR* items,
 	const SCHAR* const end = info + output_length;
 	const SCHAR* const end_buf = buffer + sizeof(buffer);
 
-	Attachment* err_att = 0;
-	Attachment* att = 0;
+	const Attachment* err_att = tdbb->getAttachment();
 	const SCHAR* q;
 
 	while (items < end_items && *items != isc_info_end) {
@@ -309,7 +324,7 @@ int INF_database_info(const SCHAR* items,
 #endif
 
 		case isc_info_attachment_id:
-			length = INF_convert(PAG_attachment_id(), buffer);
+			length = INF_convert(PAG_attachment_id(tdbb), buffer);
 			break;
 
 		case isc_info_ods_version:
@@ -322,7 +337,7 @@ int INF_database_info(const SCHAR* items,
 
 		case isc_info_allocation:
 			CCH_flush(tdbb, FLUSH_ALL, 0L);
-			length = INF_convert(PIO_max_alloc(dbb), buffer);
+			length = INF_convert(PageSpace::maxAlloc(dbb), buffer);
 			break;
 
 		case isc_info_sweep_interval:
@@ -456,14 +471,10 @@ int INF_database_info(const SCHAR* items,
 
 		case isc_info_creation_date:
 			{
-				WIN window(HEADER_PAGE);
-				Ods::header_page* header = (Ods::header_page*) 
-					CCH_FETCH(tdbb, &window, LCK_read, pag_header);
-
-				length = INF_convert(header->hdr_creation_date[0], p); 
+				const ISC_TIMESTAMP ts = dbb->dbb_creation_date.value();
+				length = INF_convert(ts.timestamp_date, p); 
 				p += length;
-				length += INF_convert(header->hdr_creation_date[1], p);
-				CCH_RELEASE(tdbb, &window);
+				length += INF_convert(ts.timestamp_time, p);
 			}
 			break;
 
@@ -540,9 +551,32 @@ int INF_database_info(const SCHAR* items,
 			}
 			break;
 
+		case isc_info_db_file_size:
+			{
+				BackupManager *bm = dbb->dbb_backup_manager;
+				length = INF_convert(bm ? bm->getPageCount() : 0, buffer);
+			}
+			break;
+
 		case isc_info_user_names:
 			// Assumes user names will be smaller than sizeof(buffer) - 1.
-			for (att = dbb->dbb_attachments; att; att = att->att_next) {
+			if (!(tdbb->getAttachment()->locksmith())) {
+				const UserId* user = tdbb->getAttachment()->att_user;
+				const char* uname = (user && user->usr_user_name.hasData()) ? user->usr_user_name.c_str() : "<Unknown>";
+				const SSHORT len = strlen(uname);
+				*p++ = len;
+				memcpy(p, uname, len);
+				info = INF_put_item(item, len + 1, buffer, info, end);
+				if (!info) {
+					if (transaction)
+						TRA_commit(tdbb, transaction, false);
+					return FALSE;
+				}
+				continue;
+			}
+			{ // scope for VC6
+			for (const Attachment* att = dbb->dbb_attachments; att; att = att->att_next)
+			{
 				if (att->att_flags & ATT_shutdown)
 					continue;
                 
@@ -562,10 +596,10 @@ int INF_database_info(const SCHAR* items,
 					}
 				}
 			}
+			} // end scope for VC6
 			continue;
 
 		case isc_info_page_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val =
 					(*err_att->att_val_errors)[VAL_PAG_WRONG_TYPE]
@@ -581,7 +615,6 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case isc_info_bpage_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val =
 					(*err_att->att_val_errors)[VAL_BLOB_INCONSISTENT]
@@ -595,7 +628,6 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case isc_info_record_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val =
 					(*err_att->att_val_errors)[VAL_REC_CHAIN_BROKEN]
@@ -613,7 +645,6 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case isc_info_dpage_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val =
 					(*err_att->att_val_errors)[VAL_DATA_PAGE_CONFUSED]
@@ -627,7 +658,6 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case isc_info_ipage_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val =
 					(*err_att->att_val_errors)[VAL_INDEX_PAGE_CORRUPT] +
@@ -643,7 +673,6 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case isc_info_ppage_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val = (*err_att->att_val_errors)[VAL_P_PAGE_LOST]
 					+
@@ -656,7 +685,6 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case isc_info_tpage_errors:
-			err_att = tdbb->tdbb_attachment;
 			if (err_att->att_val_errors) {
 				err_val = (*err_att->att_val_errors)[VAL_TIP_LOST]
 					+ (*err_att->att_val_errors)[VAL_TIP_LOST_SEQUENCE]
@@ -715,7 +743,7 @@ int INF_database_info(const SCHAR* items,
 
 		case isc_info_db_size_in_pages:
 			CCH_flush(tdbb, FLUSH_ALL, 0L);
-			length = INF_convert(PIO_act_alloc(dbb), buffer);
+			length = INF_convert(PageSpace::actAlloc(dbb), buffer);
 			break;
 
 		case isc_info_oldest_transaction:
@@ -763,7 +791,7 @@ int INF_database_info(const SCHAR* items,
 			break;
 
 		case frb_info_att_charset:
-			length = INF_convert(tdbb->tdbb_attachment->att_charset, buffer);
+			length = INF_convert(tdbb->getAttachment()->att_charset, buffer);
 			break;
 
 		default:
@@ -844,6 +872,16 @@ int INF_request_info(const jrd_req* request,
 
 	const SCHAR* const end_items = items + item_length;
 	const SCHAR* const end = info + output_length;
+	SCHAR* start_info;
+	
+	if (*items == isc_info_length) {
+		start_info = info;
+		items++;
+	}
+	else {
+		start_info = 0;
+	}
+
 	SCHAR buffer[256];
 	memset(buffer, 0, sizeof(buffer));
 	SCHAR* buffer_ptr = buffer;
@@ -968,6 +1006,14 @@ int INF_request_info(const jrd_req* request,
 
 	*info++ = isc_info_end;
 
+	if (start_info && (end - info >= 7))
+	{
+		SLONG number = info - start_info;
+		memmove(start_info + 7, start_info, number);
+		length = INF_convert(number, buffer);
+		INF_put_item(isc_info_length, length, buffer, start_info, end);
+	}
+
 	return TRUE;
 }
 
@@ -992,6 +1038,15 @@ int INF_transaction_info(const jrd_tra* transaction,
 
 	const SCHAR* const end_items = items + item_length;
 	const SCHAR* const end = info + output_length;
+	SCHAR* start_info;
+	
+	if (*items == isc_info_length) {
+		start_info = info;
+		items++;
+	}
+	else {
+		start_info = 0;
+	}
 
 	while (items < end_items && *items != isc_info_end) {
 		SCHAR item = *items++;
@@ -1065,6 +1120,14 @@ int INF_transaction_info(const jrd_tra* transaction,
 
 	*info++ = isc_info_end;
 
+	if (start_info && (end - info >= 7))
+	{
+		SLONG number = info - start_info;
+		memmove(start_info + 7, start_info, number);
+		length = INF_convert(number, buffer);
+		INF_put_item(isc_info_length, length, buffer, start_info, end);
+	}
+
 	return TRUE;
 }
 
@@ -1083,7 +1146,7 @@ static USHORT get_counts(USHORT count_id, SCHAR* buffer, USHORT length)
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
 
-	const vcl* vector = tdbb->tdbb_attachment->att_counts[count_id];
+	const vcl* vector = tdbb->getAttachment()->att_counts[count_id];
 	if (!vector)
 		return 0;
 
