@@ -100,9 +100,9 @@ void NBackupState::blockingAstHandler(thread_db* tdbb)
 	if (!backup_manager->database_flush_in_progress() && cached_lock->lck_physical == LCK_read) {
 		CCH_flush_ast(tdbb);
 		NBAK_TRACE_AST(("database FLUSHED"));
-		}
-	GlobalRWLock::blockingAstHandler(tdbb);
 	}
+	GlobalRWLock::blockingAstHandler(tdbb);
+}
 /******************************** NBACKUP ALLOCATION TABLE SYNCHRONIZER ******************************/
 NBackupAlloc::NBackupAlloc(thread_db* tdbb, MemoryPool& p, BackupManager *bakMan):
 	GlobalRWLock(tdbb, p, LCK_backup_alloc, 0, NULL)
@@ -144,7 +144,7 @@ void BackupManager::release_dirty_page(thread_db* tdbb, SLONG owner_handle)
 	if (!(tdbb->tdbb_flags & TDBB_backup_write_locked)) {
 		database_lock->unlock(tdbb, LCK_read, owner_handle);
 		NBAK_TRACE(("release_dirty_page"));
-	} 
+	}
 }
 
 void BackupManager::change_dirty_page_owner(thread_db* tdbb, SLONG from_handle, SLONG to_handle)
@@ -166,13 +166,13 @@ void BackupManager::lock_clean_database(
 	if (!database_lock->lock(tdbb, LCK_write, wait)) {
 		database_lock->flags &= ~NBAK_state_blocking;
 		ERR_bugcheck_msg("Error: can't lock clean database");
-	} 
+	}
 	NBAK_TRACE(("database write locked"));
 	database_lock->flags &= ~NBAK_state_blocking;
 	tdbb->tdbb_flags |= TDBB_backup_write_locked;
 	CCH_FETCH(tdbb, window, LCK_write, pag_header);
 }
-
+ 
 void BackupManager::unlock_clean_database(
 	thread_db* tdbb) 
 {
@@ -180,14 +180,14 @@ void BackupManager::unlock_clean_database(
 	database_lock->unlock(tdbb, LCK_write);
 
 	NBAK_TRACE(("database write unlocked"));
-}
+} 
 
 void BackupManager::lock_shared_database(thread_db* tdbb, SSHORT wait)
 {
 	if (!(tdbb->tdbb_flags & TDBB_backup_write_locked))
 		if (!database_lock->lock(tdbb, LCK_read, wait)) {
 			ERR_bugcheck_msg("Error: can't lock database on llRead");
-	} 
+		}
 }
 
 void BackupManager::unlock_shared_database(thread_db* tdbb)
@@ -203,7 +203,7 @@ void BackupManager::lock_alloc_write(thread_db* tdbb, SSHORT wait)
 		ERR_bugcheck_msg("Error: lock allocation table on write");
 	}
 	NBAK_TRACE(("lock_alloc_write"));
-	}
+}
 
 void BackupManager::unlock_alloc_write(thread_db* tdbb)
 {
@@ -215,9 +215,9 @@ void BackupManager::lock_alloc(thread_db* tdbb, SSHORT wait)
 {
 	if (!alloc_lock->lock(tdbb, LCK_read, wait)) {
 		ERR_bugcheck_msg("Error: lock allocation table on read");
-	} 
-	NBAK_TRACE(("lock_alloc"));
 	}
+	NBAK_TRACE(("lock_alloc"));
+}
 
 void BackupManager::unlock_alloc(thread_db* tdbb)
 {
@@ -239,15 +239,11 @@ void BackupManager::begin_backup(thread_db* tdbb)
 	NBAK_TRACE(("begin_backup"));
 
 	// Check for raw device
-	if ((!explicit_diff_name) && database->onRawDevice()) 
-	{
-		ERR_post(isc_random,
-			     isc_arg_string,
-				 "Difference file name should be set explicitly for database on raw device",
-				 0);
+	if ((!explicit_diff_name) && database->onRawDevice()) {
+		ERR_post(isc_need_difference, isc_arg_end);
 	}
 
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 
 	bool header_locked = false;
 	bool database_locked = false;
@@ -268,14 +264,14 @@ void BackupManager::begin_backup(thread_db* tdbb)
 			return;
 		}
 
-		// Create file
+		// Create file	
 		NBAK_TRACE(("Creating difference file %s", diff_name.c_str()));
-		diff_file = PIO_create(database, diff_name, true, false);
+		diff_file = PIO_create(database, diff_name, true, false, false);
 #ifdef UNIX 
 		// adjust difference file access rights to make it match main DB ones
 		if (diff_file && geteuid() == 0) {
 			struct stat st;
-			while (fstat(database->dbb_file->fil_desc, &st) != 0) {
+			while (fstat(database->dbb_page_manager.findPageSpace(DB_PAGE_SPACE)->file->fil_desc, &st) != 0) {
 				if (errno != EINTR) {
 					Firebird::system_call_failed::raise("fstat");
 				}
@@ -313,15 +309,16 @@ void BackupManager::begin_backup(thread_db* tdbb)
 
 		header_locked = false;
 		CCH_RELEASE(tdbb, &window);
-
+	
 		backup_state = newState;
 		current_scn = adjusted_scn;
 
 		database_locked = false;
 		unlock_clean_database(tdbb);
-		 // All changes go to the difference file now
+
+		// All changes go to the difference file now
 	}
-	catch (const std::exception&) {
+	catch (const Firebird::Exception&) {
 		backup_state = nbak_state_unknown;
 		if (header_locked)
 			CCH_RELEASE(tdbb, &window);
@@ -331,6 +328,48 @@ void BackupManager::begin_backup(thread_db* tdbb)
 	}
 }
 
+
+// Determine actual DB size (raw devices support)
+ULONG BackupManager::getPageCount()
+{
+	if (backup_state != nbak_state_stalled)
+	{
+		// calculate pages only when database is locked for backup:
+		// other case such service is just dangerous
+		return 0;
+	}
+	
+	class PioCount : public Jrd::PageCountCallback
+	{
+	private:
+	    BufferDesc temp_bdb;
+		PageSpace* pageSpace;
+
+	public:
+		explicit PioCount(Database* d)
+		{
+			fb_assert(d);
+			temp_bdb.bdb_dbb = d;
+			pageSpace = d->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+			fb_assert(pageSpace);
+		}
+		virtual void newPage(const SLONG pageNum, Ods::pag* buf)
+		{
+			temp_bdb.bdb_buffer = buf;
+			temp_bdb.bdb_page = pageNum;
+			ISC_STATUS_ARRAY status;
+			if (!PIO_read(pageSpace->file, &temp_bdb, temp_bdb.bdb_buffer, status)) 
+			{
+				Firebird::status_exception::raise(status);
+			}
+		}
+	};
+	PioCount pioCount(database);
+	
+	return PAG_page_count(database, &pioCount);
+}
+
+
 // Merge difference file to main files (if needed) and unlink() difference 
 // file then. If merge is already in progress method silently returns and 
 // does nothing (so it can be used for recovery on database startup). 
@@ -339,17 +378,17 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 	NBAK_TRACE(("end_backup, recover=%i", recover));
 
 	// Check for recover
-	Database *dbb = tdbb->tdbb_database;
+	Database *dbb = tdbb->getDatabase();
 
 	GlobalRWLock endLock(tdbb, *database->dbb_permanent, LCK_backup_end, 0, NULL, LCK_OWNER_attachment, LCK_OWNER_attachment, false);
 
 	if (!endLock.lock(tdbb, LCK_write, LCK_NO_WAIT)) {
-		// Someboby hold llWrite lock on LCK_backup_end. We need not to do end_backup
+		// Someboby holds write lock on LCK_backup_end. We need not to do end_backup
 		return;
 	}
 
 	// STEP 1. Change state in header to "merge"
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	Ods::header_page* header;
 
 	bool database_locked = false;
@@ -368,8 +407,8 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 			NBAK_TRACE(("invalid state %d", backup_state));
 			endLock.unlock(tdbb, LCK_write);
 			unlock_shared_database(tdbb);
-				return;
-			}
+			return;
+		}
 		unlock_shared_database(tdbb);
 		// Here backup state can be changed. Need to check it again after lock
 		lock_clean_database(tdbb, true, &window);
@@ -405,7 +444,7 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 			database_locked = false;
 		}
 	}
-	catch (const std::exception&) {
+	catch (const Firebird::Exception&) {
 		backup_state = nbak_state_unknown;
 		if (header_locked)
 			CCH_RELEASE(tdbb, &window);
@@ -415,7 +454,7 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 		throw;
 	}
 
-
+	
 	// STEP 2. Merging database and delta
 	// Here comes the dirty work. We need to reapply all changes from difference file to database
 	// Release write state lock and get read lock. 
@@ -431,10 +470,10 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 		actualize_alloc(tdbb);
 		NBAK_TRACE(("Merge. Alloc table is actualized."));	
 		AllocItemTree::Accessor all(alloc_table);
-
+		
 		if (all.getFirst()) {
 			do {
-				WIN window2(all.current().db_page);
+				WIN window2(DB_PAGE_SPACE, all.current().db_page);
 				NBAK_TRACE(("Merge page %d, diff=%d", all.current().db_page, all.current().diff_page));
 				Ods::pag* page = CCH_FETCH(tdbb, &window2, LCK_write, pag_undefined);
 				NBAK_TRACE(("Merge: page %d is fetched", all.current().db_page));
@@ -452,9 +491,9 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 		database_locked = false;
 
 		NBAK_TRACE(("Merging is over. Database unlocked"));
-
+		
 	}
-	catch (const std::exception&) {
+	catch (const Firebird::Exception&) {
 		endLock.unlock(tdbb, LCK_write);
 		if (database_locked)
 			unlock_shared_database(tdbb);
@@ -464,12 +503,12 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 	// STEP 3. Change state in header to "normal"
 	// We finished. We need to reflect it in our database header page
 	try {
-	window.win_page = HEADER_PAGE;
-	window.win_flags = 0;
+		window.win_page = HEADER_PAGE;
+		window.win_flags = 0;
 		lock_clean_database(tdbb, true, &window);
 		database_locked = true;
 		header = (Ods::header_page*) window.win_buffer;
-	header_locked = true;
+		header_locked = true;
 
 		// Set state in database header
 		backup_state = nbak_state_normal;
@@ -496,13 +535,13 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 			diff_file = NULL;
 		}
 		unlink(diff_name.c_str());
-
+		
 		if (database_locked)
 			unlock_clean_database(tdbb);
 		NBAK_TRACE(("backup is over"));
 		endLock.unlock(tdbb, LCK_write);
 	}
-	catch (const std::exception&) {
+	catch (const Firebird::Exception&) {
 		backup_state = nbak_state_unknown;
 		endLock.unlock(tdbb, LCK_write);
 		if (header_locked)
@@ -514,7 +553,7 @@ void BackupManager::end_backup(thread_db* tdbb, bool recover)
 
 	return;
 }
-
+	
 bool BackupManager::actualize_alloc(thread_db* tdbb)
 {
 	ISC_STATUS *status_vector = tdbb->tdbb_status_vector;
@@ -530,7 +569,7 @@ bool BackupManager::actualize_alloc(thread_db* tdbb)
 			// Difference file pointer pages have one ULONG as number of pages allocated on the page and
 			// then go physical numbers of pages from main database file. Offsets of numbers correspond
 			// to difference file pages.
-
+		
 			// Get offset of pointer page. We can do so because page sizes are powers of 2
 			temp_bdb.bdb_page = last_allocated_page & ~(database->dbb_page_size / sizeof(ULONG) - 1);
 			temp_bdb.bdb_dbb = database;
@@ -539,10 +578,10 @@ bool BackupManager::actualize_alloc(thread_db* tdbb)
 			if (!PIO_read(diff_file, &temp_bdb, temp_bdb.bdb_buffer, status_vector)) {
 				return false;
 			}
-			for (ULONG i = last_allocated_page - temp_bdb.bdb_page; i < alloc_buffer[0]; i++)
+			for (ULONG i = last_allocated_page - temp_bdb.bdb_page.getPageNum(); i < alloc_buffer[0]; i++)
 			{
-				NBAK_TRACE(("alloc item page=%d, diff=%d", alloc_buffer[i + 1], temp_bdb.bdb_page + i + 1));
-				if (!alloc_table->add(AllocItem(alloc_buffer[i + 1], temp_bdb.bdb_page + i + 1)))
+				NBAK_TRACE(("alloc item page=%d, diff=%d", alloc_buffer[i + 1], temp_bdb.bdb_page.getPageNum() + i + 1));
+				if (!alloc_table->add(AllocItem(alloc_buffer[i + 1], temp_bdb.bdb_page.getPageNum() + i + 1)))
 				{
 					database->dbb_flags |= DBB_bugcheck;
 					status_vector[0] = isc_arg_gds;
@@ -554,7 +593,7 @@ bool BackupManager::actualize_alloc(thread_db* tdbb)
 					return false;
 				}
 			}
-			last_allocated_page = temp_bdb.bdb_page + alloc_buffer[0];
+			last_allocated_page = temp_bdb.bdb_page.getPageNum() + alloc_buffer[0];
 			if (alloc_buffer[0] == database->dbb_page_size / sizeof(ULONG) - 1)
 				// if page is full adjust position for next pointer page
 				last_allocated_page++;
@@ -563,13 +602,13 @@ bool BackupManager::actualize_alloc(thread_db* tdbb)
 				break;		
 		}
 	}
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		// Handle out of memory error, etc
 		delete alloc_table;
 		Firebird::stuff_exception(status_vector, ex);
 		alloc_table = NULL;
 		last_allocated_page = 0;
-		return false;				
+		return false;
 	}
 	return true;
 }
@@ -608,7 +647,7 @@ ULONG BackupManager::allocate_difference_page(thread_db* tdbb, ULONG db_page)
 	if (!PIO_write(diff_file, &temp_bdb, (Ods::pag*)empty_buffer, status_vector)) {
 		return 0;
 	}
-
+	
 	const bool alloc_page_full = alloc_buffer[0] == database->dbb_page_size / sizeof(ULONG) - 2;
 	if (alloc_page_full) {
 		// Pointer page is full. Its time to create new one.
@@ -617,7 +656,7 @@ ULONG BackupManager::allocate_difference_page(thread_db* tdbb, ULONG db_page)
 		temp_bdb.bdb_buffer = reinterpret_cast<Ods::pag*>(empty_buffer);
 		if (!PIO_write(diff_file, &temp_bdb, (Ods::pag*)empty_buffer, status_vector)) {
 			return 0;
-	}
+		}
 	}
 
 	// Write new item to the allocation table
@@ -633,7 +672,7 @@ ULONG BackupManager::allocate_difference_page(thread_db* tdbb, ULONG db_page)
 	try {
 		alloc_table->add(AllocItem(db_page, last_allocated_page));
 	}
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		// Handle out of memory error
 		delete alloc_table;
 		alloc_table = NULL;
@@ -647,7 +686,7 @@ ULONG BackupManager::allocate_difference_page(thread_db* tdbb, ULONG db_page)
 		memset(alloc_buffer, 0, database->dbb_page_size);
 		return last_allocated_page - 1;
 	}
-
+	
 	return last_allocated_page;
 }
 
@@ -658,11 +697,13 @@ bool BackupManager::write_difference(ISC_STATUS* status, ULONG diff_page, Ods::p
 	temp_bdb.bdb_page = diff_page;
 	temp_bdb.bdb_dbb = database;
 	temp_bdb.bdb_buffer = page;
+	// Check that diff page is not allocation page
+	fb_assert(diff_page % (database->dbb_page_size / sizeof(ULONG)));
 	if (!PIO_write(diff_file, &temp_bdb, page, status))
 		return false;
 	return true;
 }
-
+	
 bool BackupManager::read_difference(thread_db* tdbb, ULONG diff_page, Ods::pag* page)
 {
 	NBAK_TRACE(("read_diff"));
@@ -672,21 +713,21 @@ bool BackupManager::read_difference(thread_db* tdbb, ULONG diff_page, Ods::pag*
 	temp_bdb.bdb_buffer = page;
 	if (!PIO_read(diff_file, &temp_bdb, page, tdbb->tdbb_status_vector))		
 		return false;
-	return true;
+	return true;	
 }
-
+	
 BackupManager::BackupManager(thread_db* tdbb, Database* _database, int ini_state) :
-	 dbCreating(false), database(_database), diff_file(NULL), alloc_table(NULL), 
-	last_allocated_page(0),	current_scn(0), diff_name(*_database->dbb_permanent), 
+	dbCreating(false), database(_database), diff_file(NULL), alloc_table(NULL),
+	last_allocated_page(0), current_scn(0), diff_name(*_database->dbb_permanent),
 	explicit_diff_name(false)
 {
 	// Allocate various database page buffers needed for operation
 	temp_buffers_space = FB_NEW(*database->dbb_permanent) BYTE[database->dbb_page_size * 3 + MIN_PAGE_SIZE];
-	// Align it at sector boundary for faster IO
+	// Align it at sector boundary for faster IO (also guarantees correct alignment for ULONG later)
 	BYTE *temp_buffers = reinterpret_cast<BYTE*>(
 		FB_ALIGN(reinterpret_cast<U_IPTR>(temp_buffers_space), MIN_PAGE_SIZE));
 	memset(temp_buffers, 0, database->dbb_page_size * 3);
-
+	
 	backup_state = ini_state;
 
 	empty_buffer = reinterpret_cast<ULONG*>(temp_buffers);
@@ -702,9 +743,7 @@ BackupManager::BackupManager(thread_db* tdbb, Database* _database, int ini_state
 void BackupManager::shutdown_locks(thread_db* tdbb)
 {
 	delete alloc_lock;
-	alloc_lock = NULL;
 	delete database_lock;
-	database_lock = NULL;
 }
 
 
@@ -717,7 +756,7 @@ BackupManager::~BackupManager()
 void BackupManager::set_difference(thread_db* tdbb, const char* filename) 
 {
 	if (filename) {
-		WIN window(HEADER_PAGE);
+		WIN window(HEADER_PAGE_NUMBER);
 		Ods::header_page* header =
 			(Ods::header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 		CCH_MARK_MUST_WRITE(tdbb, &window);
@@ -745,22 +784,25 @@ bool BackupManager::actualize_state(thread_db* tdbb)
 	}
 
 	ISC_STATUS *status = tdbb->tdbb_status_vector;
-
+			
 	// Read original page from database file or shadows.
 	SSHORT retryCount = 0;
 	Ods::header_page* header = reinterpret_cast<Ods::header_page*>(spare_buffer);
 	BufferDesc temp_bdb;
-	temp_bdb.bdb_page = HEADER_PAGE;
+	temp_bdb.bdb_page = HEADER_PAGE_NUMBER;
 	temp_bdb.bdb_dbb = database;
 	temp_bdb.bdb_buffer = reinterpret_cast<Ods::pag*>(header);
-	jrd_file* file = database->dbb_file;
+	PageSpace* pageSpace = 
+		database->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	fb_assert(pageSpace);
+	jrd_file* file = pageSpace->file;
 	while (!PIO_read(file, &temp_bdb, temp_bdb.bdb_buffer, status)) {
 		if (!CCH_rollover_to_shadow(database, file, false)) {
 			NBAK_TRACE(("Shadow change error"));
 			return false;
 		}
-		if (file != database->dbb_file)
-			file = database->dbb_file;
+		if (file != pageSpace->file)
+			file = pageSpace->file;
 		else {
 			if (retryCount++ == 3) {
 				NBAK_TRACE(("IO error"));
@@ -795,7 +837,7 @@ bool BackupManager::actualize_state(thread_db* tdbb)
 	}
 	if (!explicit_diff_name)
 		generate_filename();
-
+		
 	if (new_backup_state == nbak_state_normal || missed_cycle) {
 		if (diff_file) {
 			NBAK_TRACE(("Close difference file"));
@@ -816,9 +858,9 @@ bool BackupManager::actualize_state(thread_db* tdbb)
 	if (new_backup_state != nbak_state_normal && !diff_file) {
 		try {
 			NBAK_TRACE(("Open difference file"));
-			diff_file = PIO_open(database, diff_name, false, NULL, diff_name, false);
+			diff_file = PIO_open(database, diff_name, false, diff_name, false);
 		}
-		catch (const std::exception& ex) {
+		catch (const Firebird::Exception& ex) {
 			Firebird::stuff_exception(status, ex);
 			return false;
 		}
