@@ -85,6 +85,8 @@ public:
 	ULONG flags; 
 
 	NBackupState(thread_db* tdbb, MemoryPool& p, BackupManager *bakMan);
+	virtual ~NBackupState() { }
+
 protected:
 	BackupManager *backup_manager;
 	virtual void blockingAstHandler(thread_db* tdbb);
@@ -97,6 +99,7 @@ class NBackupAlloc: public GlobalRWLock
 {
 public:
 	NBackupAlloc(thread_db* tdbb, MemoryPool& p, BackupManager *bakMan);
+	virtual ~NBackupAlloc() { }
 
 protected:
 	BackupManager *backup_manager;
@@ -109,7 +112,7 @@ const UATOM NBAK_state_blocking		= 1;	// Changing the blocking state. All pages
 const SATOM nbak_state_normal	= 0x0;     // Normal mode. Changes are simply written to main files
 const SATOM nbak_state_stalled	= 0x400;   // 1024 Main files are locked. Changes are written to diff file
 const SATOM nbak_state_merge	= 0x800;   // 2048 Merging changes from diff file into main files
-const SATOM nbak_state_unknown	= -1;      // State is unknown. Needs to be read from disk
+const SATOM nbak_state_unknown	= -1;      // State is unknown. Needs to be read from disk 
 
 /*
  *  The functional responsibilities of NBAK are:
@@ -159,7 +162,7 @@ const SATOM nbak_state_unknown	= -1;      // State is unknown. Needs to be read
  *  
  *  To change backup state, engine takes LCK_backup_database lock first, forcing all
  *  dirty pages to the disk, modifies the header page reflecting the state change,
- *  and releases the lock allowing transation processing to continue.
+ *  and releases the lock allowing transaction processing to continue.
  *  
  *  LCK_backup_alloc is used to protect mapping table between difference file
  *  (.delta) and the database. To add new page to the mapping attachment needs to
@@ -174,6 +177,27 @@ const SATOM nbak_state_unknown	= -1;      // State is unknown. Needs to be read
 
 class BackupManager {
 public:
+	class SharedDatabaseHolder
+	{
+	public:
+		explicit SharedDatabaseHolder(thread_db* atdbb, BackupManager* bm)
+			: backupManager(bm), tdbb(atdbb)
+		{
+			backupManager->lock_shared_database(tdbb, true);
+		}
+		~SharedDatabaseHolder()
+		{
+			backupManager->unlock_shared_database(tdbb);
+		}
+	private:
+		// copying is prohibited
+		SharedDatabaseHolder(const SharedDatabaseHolder&);
+		SharedDatabaseHolder& operator =(const SharedDatabaseHolder&);
+
+		BackupManager* backupManager;
+		thread_db* tdbb;
+	};
+
 	// Set when db is creating. Default = false
 	bool dbCreating;
 
@@ -200,15 +224,15 @@ public:
 	ULONG get_current_scn() const {
 		return current_scn;
 	}
-
+	
 	// Initialize and open difference file for writing
 	void begin_backup(thread_db* tdbb);
-
+	
 	// Merge difference file to main files (if needed) and unlink() difference 
 	// file then. If merge is already in progress method silently returns false and 
 	// does nothing (so it can be used for recovery on database startup). 
 	void end_backup(thread_db* tdbb, bool recover);
-
+	
 	// Function for force all connections to flush their caches
 	// and prevent them from marking new dirty pages
 	void lock_clean_database(thread_db* tdbb, SSHORT wait, WIN* window);
@@ -232,11 +256,11 @@ public:
 
 	// Return next page index in the difference file to be allocated
 	ULONG allocate_difference_page(thread_db* tdbb, ULONG db_page);
-
+	
 	// Must have ISC_STATUS because it is called from write_page
 	bool write_difference(ISC_STATUS* status, ULONG diff_page, Ods::pag* page);
 	bool read_difference(thread_db* tdbb, ULONG diff_page, Ods::pag* page);
-
+	
 	// Routines to declare and release interest in the database file
 	void checkout_dirty_page(thread_db* tdbb, SLONG owner_handle);
 	void release_dirty_page(thread_db* tdbb, SLONG owner_handle);
@@ -260,6 +284,9 @@ public:
 	// Make appropriate information up-to-date
 	bool actualize_state(thread_db* tdbb);
 	bool actualize_alloc(thread_db* tdbb);
+	
+	// Get size (in pages) of locked database file
+	ULONG getPageCount();
 
 	// Subsystem finalization. Called from shutdown()
 	~BackupManager();
