@@ -101,6 +101,8 @@ bool SHUT_blocking_ast(Database* dbb)
 		case isc_dpb_shut_full:
 			dbb->dbb_ast_flags |= DBB_shutdown | DBB_shutdown_full;
 			break;
+		default:
+			fb_assert(false);
 		}
 					
 		dbb->dbb_shutdown_delay = 0; // not tested anywhere
@@ -142,20 +144,22 @@ bool SHUT_database(Database* dbb, SSHORT flag, SSHORT delay)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Attachment* attachment = tdbb->tdbb_attachment;
+	Attachment* attachment = tdbb->getAttachment();
 
 /* Only platform's user locksmith can shutdown or bring online
    a database. */
 
-	if (!(attachment->att_user->usr_flags & (USR_locksmith | USR_owner))) {
+	if (!attachment->locksmith()) 
+	{
 		return false;
 	}
 
+	const int shut_mode = flag & isc_dpb_shut_mode_mask;
+
 	// Check if requested shutdown mode is valid
 	// Note that if we are already in requested mode we just return true.
 	// This is required to ensure backward compatible behavior (gbak relies on that, 
 	// user-written scripts may rely on this behaviour too)
-	int shut_mode = flag & isc_dpb_shut_mode_mask;
 	switch (shut_mode) {
 	case isc_dpb_shut_full:
 		if (dbb->dbb_ast_flags & DBB_shutdown_full) 
@@ -234,7 +238,7 @@ bool SHUT_database(Database* dbb, SSHORT flag, SSHORT delay)
 		SHUT_blocking_ast(dbb);
 		attachment->att_flags &= ~ATT_shutdown_manager;
 		++dbb->dbb_use_count;
-		ERR_post(isc_shutfail, 0);
+		ERR_post(isc_shutfail, isc_arg_end);
 	}
 
 /* Once there are no more transactions active, force all remaining
@@ -242,18 +246,23 @@ bool SHUT_database(Database* dbb, SSHORT flag, SSHORT delay)
 
 	if (flag & isc_dpb_shut_transaction) {
 		exclusive = false;
-		flag = isc_dpb_shut_force | (flag & isc_dpb_shut_mode_mask);
+		flag = isc_dpb_shut_force | shut_mode;
 	}
 
 	dbb->dbb_ast_flags |= DBB_shutdown;
 	dbb->dbb_ast_flags &= ~(DBB_shutdown_single | DBB_shutdown_full);
-	switch (flag & isc_dpb_shut_mode_mask) {
+	switch (shut_mode) {
+	case isc_dpb_shut_normal:
+	case isc_dpb_shut_multi:
+		break;
 	case isc_dpb_shut_single:
 		dbb->dbb_ast_flags |= DBB_shutdown_single;
 		break;
 	case isc_dpb_shut_full:
 		dbb->dbb_ast_flags |= DBB_shutdown_full;
 		break;
+	default:
+		fb_assert(false);
 	}
 
 	if (!exclusive && (flag & isc_dpb_shut_force)) {
@@ -263,13 +272,13 @@ bool SHUT_database(Database* dbb, SSHORT flag, SSHORT delay)
 
 	++dbb->dbb_use_count;
 	dbb->dbb_ast_flags &= ~(DBB_shut_force | DBB_shut_attach | DBB_shut_tran);
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	Ods::header_page* header =
 		(Ods::header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 	CCH_MARK_MUST_WRITE(tdbb, &window);
 	// Set appropriate shutdown mode in database header
 	header->hdr_flags &= ~Ods::hdr_shutdown_mask;
-	switch (flag & isc_dpb_shut_mode_mask) {
+	switch (shut_mode) {
 	case isc_dpb_shut_normal:
 		break;
 	case isc_dpb_shut_multi:
@@ -281,12 +290,14 @@ bool SHUT_database(Database* dbb, SSHORT flag, SSHORT delay)
 	case isc_dpb_shut_full:
 		header->hdr_flags |= Ods::hdr_shutdown_full;
 		break;
+	default:
+		fb_assert(false);
 	}
 	CCH_RELEASE(tdbb, &window);
 	CCH_release_exclusive(tdbb);
 
 	}	// try
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		Firebird::stuff_exception(tdbb->tdbb_status_vector, ex);
 		return false;
 	}
@@ -327,17 +338,18 @@ bool SHUT_online(Database* dbb, SSHORT flag)
  **************************************/
 
 	thread_db* tdbb = JRD_get_thread_data();
-	Attachment* attachment = tdbb->tdbb_attachment;
+	Attachment* attachment = tdbb->getAttachment();
 
 /* Only platform's user locksmith can shutdown or bring online
    a database. */
 
-	if (!(attachment->att_user->usr_flags & (USR_locksmith | USR_owner))) {
+	if (!attachment->att_user->locksmith()) {
 		return false;
 	}
 	
+	const int shut_mode = flag & isc_dpb_shut_mode_mask;
+
 	// Check if requested shutdown mode is valid
-	int shut_mode = flag & isc_dpb_shut_mode_mask;
 	switch (shut_mode) {
 	case isc_dpb_shut_normal:
 		if (!(dbb->dbb_ast_flags & DBB_shutdown)) 
@@ -381,7 +393,7 @@ bool SHUT_online(Database* dbb, SSHORT flag)
 
 	/* Clear shutdown flag on database header page */
 
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	Ods::header_page* header = (Ods::header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 	CCH_MARK_MUST_WRITE(tdbb, &window);
 	// Set appropriate shutdown mode in database header
@@ -398,6 +410,8 @@ bool SHUT_online(Database* dbb, SSHORT flag)
 	case isc_dpb_shut_full:
 		header->hdr_flags |= Ods::hdr_shutdown_full;
 		break;
+	default:
+		fb_assert(false);
 	}
 	CCH_RELEASE(tdbb, &window);
 
@@ -412,7 +426,7 @@ bool SHUT_online(Database* dbb, SSHORT flag)
 	SHUT_blocking_ast(dbb);
 
 	}	// try
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		Firebird::stuff_exception(tdbb->tdbb_status_vector, ex);
 		return false;
 	}
@@ -421,9 +435,11 @@ bool SHUT_online(Database* dbb, SSHORT flag)
 }
 
 
-static bool bad_mode(thread_db* tdbb, bool ignore) {
+static bool bad_mode(thread_db* tdbb, bool ignore)
+{
 	if (!ignore) {
-		Database* dbb = tdbb->tdbb_database;
+		Database* dbb = tdbb->getDatabase();
+		
 		ISC_STATUS* status = tdbb->tdbb_status_vector;
 		*status++ = isc_arg_gds;
 		*status++ = isc_bad_shutdown_mode;
@@ -437,7 +453,7 @@ static bool bad_mode(thread_db* tdbb, bool ignore) {
 
 static void check_backup_state(thread_db* tdbb)
 {
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 	dbb->dbb_backup_manager->lock_shared_database(tdbb, true);
 
@@ -447,10 +463,10 @@ static void check_backup_state(thread_db* tdbb)
 			ERR_post(isc_bad_shutdown_mode,
 					 isc_arg_string,
 					 ERR_cstring(dbb->dbb_filename.c_str()),
-					 0);
+					 isc_arg_end);
 		}
 	}
-	catch(const std::exception&) {
+	catch (const Firebird::Exception&) {
 		dbb->dbb_backup_manager->unlock_shared_database(tdbb);
 		throw;
 	}
@@ -458,6 +474,7 @@ static void check_backup_state(thread_db* tdbb)
 	dbb->dbb_backup_manager->unlock_shared_database(tdbb);
 }
 
+
 static bool notify_shutdown(Database* dbb, SSHORT flag, SSHORT delay)
 {
 /**************************************
@@ -484,7 +501,7 @@ static bool notify_shutdown(Database* dbb, SSHORT flag, SSHORT delay)
 
 /* Send blocking ASTs to database users */
 
-	bool exclusive =
+	const bool exclusive =
 		CCH_exclusive(tdbb, LCK_PW, delay > 0 ? -SHUT_WAIT_TIME : -1);
 
 	if (exclusive && (delay != -1)) {
@@ -523,6 +540,8 @@ static bool shutdown_locks(Database* dbb, SSHORT flag)
 	dbb->dbb_ast_flags &= ~(DBB_shutdown | DBB_shutdown_single | DBB_shutdown_full);
 	
 	switch (flag & isc_dpb_shut_mode_mask) {
+	case isc_dpb_shut_normal:
+		break;
 	case isc_dpb_shut_multi:
 		dbb->dbb_ast_flags |= DBB_shutdown;
 		break;
@@ -532,10 +551,12 @@ static bool shutdown_locks(Database* dbb, SSHORT flag)
 	case isc_dpb_shut_full:
 		dbb->dbb_ast_flags |= DBB_shutdown | DBB_shutdown_full;
 		break;
+	default:
+		fb_assert(false);
 	}
 
 	Attachment* attachment;
-
+	
 	for (attachment = dbb->dbb_attachments; attachment;
 		 attachment = attachment->att_next)
 	{
@@ -544,11 +565,12 @@ static bool shutdown_locks(Database* dbb, SSHORT flag)
 	}
 
 	if (dbb->dbb_use_count) {
+#ifdef SUPERSERVER
 		/* Let active database threads rundown */
-
 		THREAD_EXIT();
 		THREAD_SLEEP(1 * 1000);
 		THREAD_ENTER();
+#endif
 		return false;
 	}
 
@@ -584,6 +606,8 @@ static bool shutdown_locks(Database* dbb, SSHORT flag)
 
 	if (!shut_attachment) {
 		CCH_shutdown_database(dbb);
+		if (dbb->dbb_monitor_lock)
+			LCK_release(tdbb, dbb->dbb_monitor_lock);
 		if (dbb->dbb_shadow_lock)
 			LCK_release(tdbb, dbb->dbb_shadow_lock);
 		if (dbb->dbb_retaining_lock)
