@@ -1,6 +1,6 @@
 /*
  *      PROGRAM:        JRD Lock Manager
- *      MODULE:         print.c
+ *      MODULE:         print.cpp
  *      DESCRIPTION:    Lock Table printer
  *
  * The contents of this file are subject to the Interbase Public
@@ -95,16 +95,16 @@ static const TEXT history_names[][10] = {
 
 static const UCHAR compatibility[] = {
 
-/*				Shared	Prot	Shared	Prot
-		none	null	 Read	Read	Write	Write	Exclusive */
+/*							Shared	Prot	Shared	Prot
+			none	null	Read	Read	Write	Write	Exclusive */
 
-/* none */ 1, 1, 1, 1, 1, 1, 1,
-/* null */ 1, 1, 1, 1, 1, 1, 1,
-/* SR */ 1, 1, 1, 1, 1, 1, 0,
-/* PR */ 1, 1, 1, 1, 0, 0, 0,
-/* SW */ 1, 1, 1, 0, 1, 0, 0,
-/* PW */ 1, 1, 1, 0, 0, 0, 0,
-/* EX */ 1, 1, 0, 0, 0, 0, 0
+/* none */	1,		1,		1,		1,		1,		1,		1,
+/* null */	1,		1,		1,		1,		1,		1,		1,
+/* SR */	1,		1,		1,		1,		1,		1,		0,
+/* PR */	1,		1,		1,		1,		0,		0,		0,
+/* SW */	1,		1,		1,		0,		1,		0,		0,
+/* PW */	1,		1,		1,		0,		0,		0,		0,
+/* EX */	1,		1,		0,		0,		0,		0,		0
 };
 
 #define COMPATIBLE(st1, st2)	compatibility [st1 * LCK_max + st2]
@@ -126,7 +126,7 @@ int CLIB_ROUTINE main( int argc, char *argv[])
  **************************************/
 	OUTFILE outfile = stdout;
 
-/* Perform some special handling when run as an Interbase service.  The
+/* Perform some special handling when run as a Firebird service.  The
    first switch can be "-svc" (lower case!) or it can be "-svc_re" followed
    by 3 file descriptors to use in re-directing stdin, stdout, and stderr. */
 
@@ -259,8 +259,7 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 					}
 				if (!sw_interactive)
 					sw_interactive =
-						(SW_I_ACQUIRE | SW_I_OPERATION | SW_I_TYPE |
-						 SW_I_WAIT);
+						(SW_I_ACQUIRE | SW_I_OPERATION | SW_I_TYPE | SW_I_WAIT);
 				sw_nobridge = true;
 				sw_seconds = sw_intervals = 1;
 				if (argc > 1) {
@@ -316,12 +315,10 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 	shmem_data.sh_mem_semaphores = 0;
 #endif
 
-	SLONG LOCK_size_mapped = DEFAULT_SIZE;
-
-#ifdef UNIX
-	LOCK_size_mapped = 0;		/* Use length of existing segment */
+#ifndef VMS
+	SLONG LOCK_size_mapped = 0;			/* Use length of existing segment */
 #else
-	LOCK_size_mapped = DEFAULT_SIZE;	/* length == 0 not supported by all non-UNIX */
+	SLONG LOCK_size_mapped = 1048576;	/* length == 0 not supported by VMS */
 #endif
 
 	ISC_STATUS_ARRAY status_vector;
@@ -375,8 +372,15 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 	if ((LOCK_header->lhb_version != SS_LHB_VERSION) &&
 		(LOCK_header->lhb_version != CLASSIC_LHB_VERSION))
 	{
-		FPRINTF(outfile, "\tUnable to read lock table version %d.\n",
+		if (LOCK_header->lhb_type == 0 && LOCK_header->lhb_version == 0) 
+		{
+			FPRINTF(outfile, "\tLock table is empty.\n");
+		}
+		else 
+		{
+			FPRINTF(outfile, "\tUnable to read lock table version %d.\n",
 				LOCK_header->lhb_version);
+		}
 		exit(FINI_OK);
 	}
 
@@ -422,7 +426,7 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 #ifdef MANAGER_PROCESS
 	int manager_pid = 0;
 	if (LOCK_header->lhb_manager) {
-		OWN manager = (OWN) SRQ_ABS_PTR(LOCK_header->lhb_manager);
+		own* manager = (own*) SRQ_ABS_PTR(LOCK_header->lhb_manager);
 		manager_pid = manager->own_process_id;
 	}
 
@@ -498,9 +502,9 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 	}
 
 	prt_que(outfile, LOCK_header, "\tOwners", &LOCK_header->lhb_owners,
-			OFFSET(OWN, own_lhb_owners));
+			OFFSET(own*, own_lhb_owners));
 	prt_que(outfile, LOCK_header, "\tFree owners",
-			&LOCK_header->lhb_free_owners, OFFSET(OWN, own_lhb_owners));
+			&LOCK_header->lhb_free_owners, OFFSET(own*, own_lhb_owners));
 	prt_que(outfile, LOCK_header, "\tFree locks",
 			&LOCK_header->lhb_free_locks, OFFSET(LBL, lbl_lhb_hash));
 	prt_que(outfile, LOCK_header, "\tFree requests",
@@ -527,7 +531,7 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 		SRQ_PTR least_owner_id = 0x7FFFFFFF;
 		const srq* least_owner_ptr = &LOCK_header->lhb_owners;
 		SRQ_LOOP(LOCK_header->lhb_owners, que_inst) {
-			OWN this_owner = (OWN) ((UCHAR*) que_inst - OFFSET(OWN, own_lhb_owners));
+			own* this_owner = (own*) ((UCHAR*) que_inst - OFFSET(own*, own_lhb_owners));
 			if (SRQ_REL_PTR(this_owner) < least_owner_id) {
 				least_owner_id = SRQ_REL_PTR(this_owner);
 				least_owner_ptr = que_inst;
@@ -537,14 +541,14 @@ int CLIB_ROUTINE main( int argc, char *argv[])
 		do {
 			if (que_inst != &LOCK_header->lhb_owners)
 				prt_owner(outfile, LOCK_header,
-						  (OWN) ((UCHAR*) que_inst - OFFSET(OWN, own_lhb_owners)),
+						  (own*) ((UCHAR*) que_inst - OFFSET(own*, own_lhb_owners)),
 						  sw_requests, sw_waitlist);
 			que_inst = SRQ_NEXT((*que_inst));
 		} while (que_inst != least_owner_ptr);
 #else
 		SRQ_LOOP(LOCK_header->lhb_owners, que_inst) {
 			prt_owner(outfile, LOCK_header,
-					  (OWN) ((UCHAR*) que_inst - OFFSET(OWN, own_lhb_owners)),
+					  (own*) ((UCHAR*) que_inst - OFFSET(own*, own_lhb_owners)),
 					  sw_requests, sw_waitlist);
 		}
 #endif /* SOLARIS_MT */
@@ -903,30 +907,56 @@ static void prt_lock(
 			lock->lbl_series, lock->lbl_parent, lock->lbl_state,
 			lock->lbl_size, lock->lbl_length, lock->lbl_data);
 
-	if (lock->lbl_length == 4) {
-		SLONG key;
-		UCHAR* p = (UCHAR *) &key;
+	if ((lock->lbl_series == Jrd::LCK_bdb || lock->lbl_series == Jrd::LCK_btr_dont_gc) && 
+		lock->lbl_length == Jrd::PageNumber::getLockLen()) 
+	{
+		// Since fb 2.1 lock keys for page numbers (series == 3) contains
+		// page space number in high long of two-longs key. Lets print it
+		// in <page_space>:<page_number> format
 		const UCHAR* q = lock->lbl_key;
-		for (const UCHAR* const end = q + 4; q < end; q++)
-			*p++ = *q;
+		
+		SLONG key;
+		memcpy(&key, q, sizeof(SLONG));
+		q += sizeof(SLONG);
+
+		ULONG pg_space;
+		memcpy(&pg_space, q, sizeof(SLONG));
+
+		FPRINTF(outfile, "\tKey: %04"ULONGFORMAT":%06"SLONGFORMAT",", pg_space, key);
+	}
+	else if (lock->lbl_length == 4) {
+		SLONG key;
+		memcpy(&key, lock->lbl_key, 4);
+
 		FPRINTF(outfile, "\tKey: %06"SLONGFORMAT",", key);
 	}
 	else {
 		UCHAR temp[512];
+		fb_assert(sizeof(temp) >= lock->lbl_length + 1); // Not enough, see <%d> below.
 		UCHAR* p = temp;
+		const UCHAR* end_temp = p + sizeof(temp) - 1;
   		const UCHAR* q = lock->lbl_key;
   		const UCHAR* const end = q + lock->lbl_length;
-		for (; q < end; q++) {
+		for (; q < end && p < end_temp; q++) {
 			const UCHAR c = *q;
 			if ((c >= 'a' && c <= 'z') ||
 				(c >= 'A' && c <= 'Z') || (c >= '0' && c <= '9') || c == '/')
 			{
 				*p++ = c;
 			}
-			else {
-				sprintf((char*) p, "<%d>", c);
-				while (*p)
-					p++;
+			else
+			{
+				char buf[6] = "";
+				int n = sprintf(buf, "<%d>", c);
+				if (n < 1 || p + n >= end_temp)
+				{
+					while (p < end_temp)
+						*p++ = '.';
+
+					break;
+				}
+				memcpy(p, buf, n);
+				p += n;
 			}
 		}
 		*p = 0;
@@ -1010,6 +1040,7 @@ static void prt_owner(OUTFILE outfile,
 		FPRINTF(outfile, " %s", (tmp & OWN_signal) ? "sgnl" : "    ");
 		FPRINTF(outfile, " %s", (tmp & OWN_wakeup) ? "wake" : "    ");
 		FPRINTF(outfile, " %s", (tmp & OWN_scanned) ? "scan" : "    ");
+		FPRINTF(outfile, " %s", (tmp & OWN_timeout) ? "tout" : "    ");
 		FPRINTF(outfile, "\n");
 	}
 
@@ -1129,7 +1160,7 @@ static void prt_owner_wait_cycle(
 				if (COMPATIBLE(owner_request->lrq_requested, lock_request->lrq_state)) 
 					continue;
 			}
-			const own* lock_owner = (OWN) SRQ_ABS_PTR(lock_request->lrq_owner);
+			const own* lock_owner = (own*) SRQ_ABS_PTR(lock_request->lrq_owner);
 			prt_owner_wait_cycle(outfile, LOCK_header, lock_owner, indent + 4,
 								 waiters);
 		}
