@@ -32,6 +32,9 @@
 #include "../jrd/gdsassert.h"
 #include "../qli/mov_proto.h"
 
+using MsgFormat::SafeArg;
+
+
 static void date_error(const TEXT*, const USHORT);
 static double double_from_text(const dsc* desc);
 static void timestamp_to_text(SLONG[2], DSC *);
@@ -384,7 +387,7 @@ int MOVQ_decompose(const TEXT* string, USHORT length, SLONG* return_value)
 		else if (*p == '.')
 			if (fraction) {
 				MOVQ_terminate(string, temp, length, sizeof(temp));
-				ERRQ_error(411, temp, NULL, NULL, NULL, NULL);
+				ERRQ_error(411, temp);
 			}
 			else
 				fraction = true;
@@ -396,7 +399,7 @@ int MOVQ_decompose(const TEXT* string, USHORT length, SLONG* return_value)
 			break;
 		else if (*p != ' ') {
 			MOVQ_terminate(string, temp, length, sizeof(temp));
-			ERRQ_error(411, temp, NULL, NULL, NULL, NULL);
+			ERRQ_error(411, temp);
 		}
 	}
 
@@ -418,7 +421,7 @@ int MOVQ_decompose(const TEXT* string, USHORT length, SLONG* return_value)
 				continue;
 			else if (*p != ' ') {
 				MOVQ_terminate(string, temp, length, sizeof(temp));
-				ERRQ_error(411, temp, NULL, NULL, NULL, NULL);
+				ERRQ_error(411, temp);
 			}
 		}
 		if (sign)
@@ -469,9 +472,7 @@ void MOVQ_fast(const SCHAR* from,
  **************************************/
 
 	if (length)
-		do {
-			*to++ = *from++;
-		} while (--length);
+		memcpy(to, from, length);
 }
 
 
@@ -520,7 +521,7 @@ double MOVQ_get_double(const dsc* desc)
 
 // Last, but not least, adjust for scale
 
-	SSHORT scale = desc->dsc_scale;
+	int scale = desc->dsc_scale;
 	if (scale == 0)
 		return value;
 
@@ -620,9 +621,9 @@ SLONG MOVQ_get_long(const dsc* desc, SSHORT scale)
 		return value;
 
 	if (scale > 0) {
-		if ((desc->dsc_dtype == dtype_short)
-			|| (desc->dsc_dtype == dtype_long)) {
-			SSHORT fraction = 0;
+		if ((desc->dsc_dtype == dtype_short) || (desc->dsc_dtype == dtype_long))
+		{
+			int fraction = 0;
 			do {
 				if (scale == 1)
 					fraction = value % 10;
@@ -728,9 +729,8 @@ if (((ALT_DSC*) from)->dsc_combined_type == ((ALT_DSC*) to)->dsc_combined_type)
 */
 	{
 		if (length)
-			do {
-				*p++ = *q++;
-			} while (--length);
+			memcpy(p, q, length);
+			
 		return;
 	}
 
@@ -968,7 +968,7 @@ static void date_error(const TEXT* string, const USHORT length)
 	SCHAR temp[128];
 
 	MOVQ_terminate(string, temp, length, sizeof(temp));
-	ERRQ_error(56, temp, NULL, NULL, NULL, NULL);
+	ERRQ_error(56, temp);
 	// Msg 56 Error converting string \"%s\" to date
 }
 
@@ -977,7 +977,7 @@ static double double_from_text(const dsc* desc)
 {
 	const TEXT* p;
 	const SSHORT length = MOVQ_get_string(desc, &p, 0, 0);
-	SSHORT scale = 0;
+	int scale = 0;
 	bool fraction = false, sign = false;
 	double value = 0;
 	const TEXT* const end = p + length;
@@ -1012,13 +1012,14 @@ static double double_from_text(const dsc* desc)
 
 	if (p < end) {
 		sign = false;
-		SSHORT exp = 0;
+		int exp = 0;
 		for (p++; p < end; p++) {
 			if (DIGIT(*p))
 				exp = exp * 10 + *p - '0';
 			else if (*p == '-' && !exp)
 				sign = true;
-			else if (*p == '+' && !exp);
+			else if (*p == '+' && !exp)
+				;
 			else if (*p != ' ')
 				IBERROR(54);	// Msg 54 conversion error
 		}
@@ -1175,7 +1176,7 @@ static void mover_error( int pattern, USHORT in_type, USHORT out_type)
  **************************************/
 	TEXT in_name[25], out_name[25], msg_unknown[40];
 
-	ERRQ_msg_get(504, msg_unknown);	// Msg504 unknown datatype %d
+	ERRQ_msg_get(504, msg_unknown, sizeof(msg_unknown));	// Msg504 unknown datatype %d
 
 	const TEXT* in = type_name(in_type);
 	if (!in) {
@@ -1189,7 +1190,7 @@ static void mover_error( int pattern, USHORT in_type, USHORT out_type)
 		sprintf(out_name, msg_unknown, out_type);
 	}
 
-	ERRQ_error(pattern, in, out, NULL, NULL, NULL);
+	ERRQ_error(pattern, SafeArg() << in << out);
 }
 
 
@@ -1359,7 +1360,7 @@ static void string_to_date(const TEXT* string, USHORT length, SLONG date[2])
 	const time_t clock = time(0);
 	tm* today = localtime(&clock);
 
-	USHORT i;
+	int i;
 	USHORT components[7];
 	for (i = 0; i < 7; i++)
 		components[i] = 0;
@@ -1525,7 +1526,7 @@ static void string_to_time(const TEXT* string, USHORT length, SLONG date[2])
 	const time_t clock = time(0);
 	const tm* today = localtime(&clock);
 
-	USHORT i;
+	int i;
 	USHORT components[7];
 	for (i = 0; i < 7; i++)
 		components[i] = 0;
