@@ -20,12 +20,13 @@
  * All Rights Reserved.
  * Contributor(s): ______________________________________.
  *
+ *
  * 2002.10.27 Sean Leyne - Completed removal of obsolete "DELTA" port
  * 2002.10.27 Sean Leyne - Completed removal of obsolete "IMP" port
  *
  * 2002.10.29 Sean Leyne - Removed obsolete "Netware" port
  *
- */
+*/
 
 #include "firebird.h"
 #include <stdio.h>
@@ -35,6 +36,7 @@
 
 #include "../jrd/common.h"
 #include "../jrd/isc.h"
+#include "../jrd/isc_s_proto.h"
 #include "../lock/lock.h"
 #include "../lock/lock_proto.h"
 #include "../jrd/license.h"
@@ -239,7 +241,7 @@ static void remove_resource(
 		&shmem_data))
 #else
 		(status_vector, expanded_filename, dummy_init, 0, shm_length,
-		&shmem_data))
+		 &shmem_data))
 #endif
 	{
 		printf("\n***Unable to access %s resources:\n", label);
@@ -361,7 +363,7 @@ static int sem_exclusive( SLONG key, SLONG count)
  **************************************/
 	int semid;
 
-#if !(defined sun || defined LINUX || defined FREEBSD || defined NETBSD || defined SINIXZ)
+#if !(defined sun || defined LINUX || defined FREEBSD || defined NETBSD)
 	return semget(key, (int) count, IPC_EXCL);
 #else
 	if ((semid = semget(key, (int) count, IPC_EXCL)) != -1)
